/***************************************************************************
;
;    This module is presented here only to serve as a sample for
;    8051 program development on the MINI-MAX/51 system.
;    BiPOM Electronics does not guarantee its functionality
;    with other micro-controller systems.
;    Please e-mail any questions and suggestions to info@bipom.com.
;    Visit BiPOM Electronics webpage at www.bipom.com for 8051 code examples.
;
;    Module: MM51DIAG.C
;
;    Revision: 1.01
;
;    Description: Performs diagnostics on the MINI-MAX/51-C board by accessing
;                 various on-board peripherals.
;
;    This program is written using Micro-C Compiler from
;    Dunfield Development Systems.
;
*/

		
#include <8051io.h>
#include <8051bit.h>            /* Bit set/clear macros */
#include <8051reg.h>


/*-----------DEFINES-----------------------------------------------------*/

#define UBYTE   unsigned char
#define USHORT  unsigned short
#define ERRCODE unsigned char

#define SUCCESS     0


#define SCL      P1.6        /* I2C clock */
#define SDA      P1.7        /* I2C data */


#define I2C_DEVICE_24CXX        0xA0
#define I2C_WRITE                  0
#define I2C_READ                   1


main()
{
    ERRCODE ec;

    register unsigned temp;

    unsigned i;
    unsigned value;

    /* Set the serial port to 9600 Baud */
    serinit(9600);

startTests:

    
    /*************** TEST RS232 SERIAL PORT ********************/

    /* Send all printable ASCII characters */
    for( i=32; i<128; i++ )
    {        
        putch( i );

        if( chkch() != 0 )
            break;
    }

    /*************** TEST I/O PORTS ********************/

    for( ;; )
    {
        for( i=0; i<255; i++ )
        {
            P1 = i | 0xC0;  /* Do not change I2C lines */
            P3 = i | 0x03;  /* Keep serial port pins untouched */
        }

        if( chkch() != 0 )
            break;
    }


    /********************* TEST EEPROM *************************/

    /* Write increasing numbers to increasing EEPROM addresses */
    for( i=0; i<256; i++ )
    {    
		ec = WriteByte( 0, i, i );
	
	        if(  ec != SUCCESS )
	            putchr( 'E' );
			else
			    putchr( 'W' );

		/* Small delay after write for physical memory write to complete */
        delay(10);

        /* Stop writing if a key is pressed */
        if( chkch() != 0 )
            break;

    }

    /* Read the numbers back from EEPROM and verify to see if they 
     * were written correctly.
     */
    for( i=0; i<256; i++ )
    {
		ec = ReadByte( 0, i, &temp );
	
        if( ec != SUCCESS )
            putchr( 'E' );
        else if( (UBYTE)temp != (i&0xFF) )
            putchr( 'V' );
		else
	    	putchr( 'R' );
    }

    /* Wait forever until a key is pressed; then start the tests */
    getch();

    goto startTests;    
}




/*********************************************************************
*                                                                   
*      WriteByte - Writes a data byte to the specified address
*                 in EEPROM.
*
*      Inputs:  UBYTE  - device number on the bus
*               unsigned       - 16-bit address
*               UBYTE - data byte to write
*
*      Outputs -  Address to EEPROM
*
*      Returns -  ERRCODE - SUCCESS if operation successful
*                           non-zero error code otherwise
*                                                                                       
*********************************************************************/

WriteByte( deviceNumber, address, data )
UBYTE deviceNumber;
unsigned  address;
UBYTE data;
{
    ERRCODE ec;
    unsigned temp;

    ec = Start();

    if( ec != SUCCESS )
        return ec;

    ec = Transmit( I2C_DEVICE_24CXX | I2C_WRITE | (deviceNumber << 1) );

    if( ec != SUCCESS )
        goto func_end;

    ec = Transmit( address & 0xFF );

    if( ec != SUCCESS )
        goto func_end;

    ec = Transmit( data );

    if( ec != SUCCESS )
        goto func_end;

func_end:

    Stop();

    return ec;
}


/*********************************************************************
*                                                                   
*      ReadByte - Reads a data byte from the specified address
*                 in EEPROM.
*
*      Inputs:  UBYTE  - device number on the bus
*               unsigned       - 16-bit address
*               register unsigned* - where to put received byte
*
*      Outputs -  Address to EEPROM
*
*      Returns -  ERRCODE - SUCCESS if operation successful
*                           non-zero error code otherwise
*                                                                                       
*********************************************************************/

ReadByte( deviceNumber, address, rcvByte )
UBYTE deviceNumber;
unsigned  address;
register unsigned* rcvByte;
{
    ERRCODE ec;
    unsigned temp;

    ec = Start();

    if( ec != SUCCESS )
        return ec;

    ec = Transmit( I2C_DEVICE_24CXX | I2C_WRITE | (deviceNumber << 1) );

    if( ec != SUCCESS )
        goto func_end;

    ec = Transmit( address & 0xFF );

    if( ec != SUCCESS )
        goto func_end;

    ec = ReadCurrent( deviceNumber, &temp );

    *rcvByte = temp;

    return ec;

func_end:

    Stop();

    return ec;
}



/*********************************************************************
*                                                                   
*      ReadCurrent - Reads a data byte from the specified address
*                 in EEPROM.
*
*      Inputs:  unsigned char  - device number on the bus
*               unsigned       - 16-bit address
*               unsigned char* - where to put error code
*
*      Outputs -  Address to EEPROM
*
*      Returns -  data byte
*                                                                                       
*********************************************************************/

ReadCurrent( deviceNumber, rcvByte )
unsigned char deviceNumber;
register unsigned* rcvByte;
{
    ERRCODE ec;
    unsigned temp;

    ec = Start();

    if( ec != SUCCESS )
        return ec;

    ec = Transmit( I2C_DEVICE_24CXX | I2C_READ | (deviceNumber << 1) );

    if( ec != SUCCESS )
        goto func_end;

    temp = Receive();

    *rcvByte = temp;

    Nak();

func_end:

    Stop();

    return ec;
}



/*
; Function: Transmit
;
; Description: Transmits a byte to the EEPROM with 
;			   most significant bit first.
;
; Inputs: 	 A - Contains the byte to be transmitted.
;			 SCL and SDA should be low on entry.
;
; Returns:	 - Carry is set if EEPROM can not acknowledge.
;			 - A is modified.
*/
Transmit( data )
UBYTE data;
{
    asm { 
        MOV R0,#-5
        LCALL ?auto0
        MOV A,[R0]      Address 

        PUSH B
        MOV B,#8       loop counter

XmitLoop

        RLC A
        MOV SDA,C
        NOP
        NOP
        SETB SCL
        NOP
        NOP
        NOP
        NOP
        CLR SCL
        DJNZ B,XmitLoop

        SETB SDA
        NOP
        NOP
        SETB SCL
        NOP
        NOP
        NOP
        NOP

        MOV C,SDA
        CLR SCL

        MOV A,#0
        RLC A

        POP B
        RET
        }
}



/*
; Function: Receive
;
; Description: Receives a byte from the EEPROM with 
;			   most significant bit first.
;
; Inputs: 	 A - Contains the byte to be transmitted.
;			 SCL should be low on entry.
;
; Returns:	 SCL is reset upon return.
;			 A contains the received byte.
*/
Receive()
{
    asm { 
        SETB SDA        SDA is an input

        PUSH B
        MOV B,#8       loop counter

RcvLoop
		
        NOP
        NOP
        NOP
        SETB SCL
        NOP
        NOP
        MOV C,SDA
        RLC A
        CLR SCL
        DJNZ B,RcvLoop

        POP B
        RET
        }
}


/*
; Function: Nak
;
; Description: Signals a negative acknowledgement 
;			   by clocking data (SDA) high
;
; Inputs: 	 SCL should be low upon entry
;
; Returns:	 SCL is low; SDA is high upon return.
*/
Nak()
{
    asm { 
        SETB SDA        negative acknowledgement 
        NOP             small delay for data setup
        NOP
        NOP
        SETB SCL        clock high
        NOP             small delay for clock setup
        NOP			
        NOP
        NOP			
        CLR SCL         clock low
        RET
        }
}


/*
; Function: Ack
;
; Description: Signals an acknowledgement 
;			   by clocking data (SDA) low
;
; Inputs: 	 SCL should be low upon entry
;
; Returns:	 SCL and SDA are high upon return.
*/
Ack()
{
    asm { 
        CLR SDA         acknowledgement 
        NOP             small delay for data setup
        NOP
        NOP
        SETB SCL        clock high
        NOP             small delay for clock setup
        NOP			
        NOP
        NOP			
        CLR SCL         clock low
        RET
        }
}


/*
; Function: Start
;
; Description: Signals a Start which is defined as negative going transition
;			   of SDA while SCL is high.
;
; Inputs: 	 None
;
; Returns:	 - Carry is set if bus is not available.
;			 - Carry is reset if success.
;			 - SCL and SDA are reset upon return.
*/
Start()
{
    asm {
        MOV A,#0        reset error flag

        SETB SDA
        SETB SCL

*       Verify bus available.

        JNB SDA,StartFail
        JNB SCL,StartFail
	        
        NOP
        CLR SDA
        NOP
        NOP
        NOP
        NOP
        NOP
        CLR SCL

        CLR C           success
        SJMP StartExit

StartFail

        MOV A,#1      error flag

StartExit

        RET
        }
}

/*
; Function: Stop
;
; Description: Signals a Stop which is defined as positive going transition
;			   of SDA while SCL is high.
;
; Inputs: 	 SCL should be reset on entry
;
; Returns:	 SCL and SDA are set upon return.
*/
Stop()
{
    asm { 
        CLR SDA
        NOP
        NOP
        SETB SCL
        NOP
        NOP
        NOP
        NOP
        NOP
        SETB SDA
        RET
        }
}



/*EOF*/
